#!/bin/bash
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: 
#      Akos Frohner <Akos.Frohner@cern.ch>
#

TEST_MODULE='org.glite.data.delegation-cli'
TEST_REQUIRES='glite-delegation-destroy glite-delegation-init glite-delegation-info voms-proxy-info'
TEST_VERBOSE=yes
#WEB_SERVICE_HOST=vtb-generic-85.cern.ch
WEB_SERVICE_HOST=lxbrb1410.cern.ch

WEB_SERVICE_ENDPOINT="https://$WEB_SERVICE_HOST:8443/glite-data-transfer-fts/services/gridsite-delegation"

if [ -z "$GLITE_LOCATION" ]; then
    GLITE_LOCATION=$(dirname $0)/../../stage
fi

# Locate test tools
if [ -d "$GLITE_LOCATION/share/test/utils/" ]; then
    TEST_UTILS_LOCATION="$GLITE_LOCATION/share/test/utils/"
elif [ -d "$(dirname $0)/../../org.glite.data.test-utils" ]; then
    TEST_UTILS_LOCATION="$(dirname $0)/../../org.glite.data.test-utils"
else
    echo "Error: Test tool directory (org.glite.data.test-utils) not found!" >&2
    exit -1
fi

source "$TEST_UTILS_LOCATION/shunit"

TEST_CERT_DIR=$GLITE_LOCATION/share/test/certificates
if [ ! -d "$TEST_CERT_DIR" ]; then
    echo "Error: Certificates directory '$TEST_CERT_DIR' was not found!" >&2
    exit -1
fi

#export X509_CERT_DIR=$TEST_CERT_DIR/grid-security/certificates
#export X509_VOMS_DIR=$TEST_CERT_DIR/grid-security/vomsdir
#export X509_USER_PROXY=$TEST_CERT_DIR/home/voms-acme.pem

if [ 'yes' = "$TEST_VERBOSE" ]; then
    echo "#Info: the test is using the following credentials:"
    echo "  export X509_USER_PROXY=$X509_USER_PROXY"
    echo "  export X509_CERT_DIR=$X509_CERT_DIR"
    echo "  export X509_VOMS_DIR=$X509_VOMS_DIR"
    echo ""
fi

export GLITE_SD_VO='org.example.single'
export GLITE_SD_PLUGIN='file'
export GLITE_SD_SERVICES_XML=$(dirname $0)/services.xml

if [ 'yes' = "$TEST_VERBOSE" ]; then
    echo "#Info: the test is using the following service-discovery settings:"
    echo "  export GLITE_SD_VO='org.example.single'"
    echo "  export GLITE_SD_PLUGIN='file'"
    echo "  export GLITE_SD_SERVICES_XML=$(dirname $0)/services.xml"
fi

#test_success "Endpoint: http.*://${WEB_SERVICE_HOST}:8.*/glite-security-delegation-service-java" glite-sd-query -t org.glite.Delegation

function test_zero {
	echo "========================================"
    echo "Cleaning up the service, etc."
    echo "========================================"
    # just to clean up the service, in case there is already a delegated credential
    # it is expected to fail, but it will do it quietly...
    glite-delegation-destroy -q -s "$WEB_SERVICE_ENDPOINT"
    test_success 'credential is delegated' glite-delegation-init -v -s "$WEB_SERVICE_ENDPOINT"
    glite-delegation-destroy -q -s "$WEB_SERVICE_ENDPOINT"
}

function test_basic {
    echo "========================================"
    echo "Basic functionality tests"
    echo "========================================"
    # just to clean up the service, in case there is already a delegated credential
    # it is expected to fail, but it will do it quietly...
    glite-delegation-destroy -q

    test_success 'credential is delegated' glite-delegation-init -v -s $WEB_SERVICE_ENDPOINT

    test_failure 'Error.*already exists'   glite-delegation-init -v -s $WEB_SERVICE_ENDPOINT

    # forcing should come over temporary problems
    test_success 'credential is delegated' glite-delegation-init -v -f -s $WEB_SERVICE_ENDPOINT

    test_success 'has been destroyed'      glite-delegation-destroy -v -s $WEB_SERVICE_ENDPOINT

    # start cleanly
    test_success 'credential is delegated' glite-delegation-init -v -s $WEB_SERVICE_ENDPOINT

    test_success 'Expiration time of '     glite-delegation-info -v -s $WEB_SERVICE_ENDPOINT
}

function test_33641 {
    # just to clean up the service, in case there is already a delegated credential
    # it is expected to fail, but it will do it quietly...
    glite-delegation-destroy -v -s $WEB_SERVICE_ENDPOINT

    test_success 'credential for B is delegated' test-glite-delegation-bug-33641 -v -f -s $WEB_SERVICE_ENDPOINT

    test_success 'Expiration time of '     glite-delegation-info -v -s $WEB_SERVICE_ENDPOINT

    #test_success 'has been destroyed'      glite-delegation-destroy -v -s $WEB_SERVICE_ENDPOINT
}

function test_fs {
    store=/tmp/dlgee_delegationStorage
    if [ -d "$store" ]; then
        echo "========================================"
        echo " Testing the filesystem backend directly"
        echo "========================================"
        # starting clean
        glite-delegation-destroy -q $WEB_SERVICE_ENDPOINT
        test_success 'credential is delegated' glite-delegation-init -v -s $WEB_SERVICE_ENDPOINT

        # comparing the storage with reality
        echo "Checking the current certificate for VOMS attributes"
        test_success 'attribute : /org.acme'   voms-proxy-info -all

        subject_clear="/C=UG/L=Tropic/O=Utopia/OU=Relaxation/CN=$LOGNAME"
        subject_urlenc="C%3DUG%2CL%3DTropic%2CO%3DUtopia%2COU%3DRelaxation%2CCN%3D$LOGNAME"
        storedir=$(ls -d $store/$subject_urlenc/* | head -1)
        proxy=$storedir/userproxy.pem

        echo "Checking the current certificate for VOMS attributes"
        test_success 'attribute : /org.acme'   voms-proxy-info -file $proxy -all
        test_success '/org.acme' cat $storedir/voms.attributes

        test_success '/C=UG/L=Tropic/O=Utopia/OU=Relaxation' \
            openssl x509 -in $proxy -noout -subject

        echo ""
        dtimeraw=$(glite-delegation-info -q)
        dtime=$(date --utc +%Y-%m-%dT%H:%M:%S --date="$dtimeraw")
        echo "Info: expiration time, according to the service: $dtime ($dtimeraw)"

        ptimeraw=$(openssl x509 -in $proxy -noout -enddate)
        ptime=$(date --utc +%Y-%m-%dT%H:%M:%S --date="${ptimeraw:9}")
        echo "Info: expiration time of the delegated proxy: $ptime ($ptimeraw)"

        ftimeraw=$(cat $storedir/termination.time)
        ftime=$(date --utc +%Y-%m-%dT%H:%M:%S --date="$ftimeraw")
        echo "Info: expiration time in the storage: $ftime ($ftimeraw)"

        test_success $ptime echo $dtime
        test_success $ptime echo $ftime
        test_success $ftime echo $dtime
    fi # end of FS-back-end test
}

function test_second {
    echo ""
    echo "A second delegation using the 'Second' ID:"
    test_success 'credential is delegated' glite-delegation-init -v Second -s $WEB_SERVICE_ENDPOINT
    test_success 'has been destroyed'      glite-delegation-destroy -v Second -s $WEB_SERVICE_ENDPOINT

    test_success 'has been destroyed'      glite-delegation-destroy -v -s $WEB_SERVICE_ENDPOINT
}

function test_voms {
    echo ""
    echo "Testing if the VOMS attributes are recognized."

    glite-delegation-destroy -q $WEB_SERVICE_ENDPOINT
    
    export X509_USER_PROXY=$TEST_CERT_DIR/home/voms-acme.pem
    test_success 'credential is delegated' glite-delegation-init -v $WEB_SERVICE_ENDPOINT

    export X509_USER_PROXY=$TEST_CERT_DIR/home/voms-coyote.pem
    test_success 'credential is delegated' glite-delegation-init -v $WEB_SERVICE_ENDPOINT
}

test_zero
test_basic
test_second
test_33641
#test_fs
#test_voms

#test_summary
